/*
 Copyright 2006 Opera Software ASA. All rights reserved.
 
This file contains Original Code and/or Contributions to the Original 
Code as defined in the Opera Web Applications License (the License). 
You may not use this file except in compliance with the License. Please 
obtain a copy of the License at http://www.opera.com/license/owal/
and read it before using this file. 

The Original Code and all Contributions to the Original Code distributed 
under the License are distributed on an AS IS basis, WITHOUT WARRANTY 
OF ANY KIND, EITHER EXPRESS OR IMPLIED, AND OPERA HEREBY DISCLAIMS ALL 
SUCH WARRANTIES, INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF 
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT. 

Please see the License for the specific language governing rights and 
limitations under the License. 
*/
var O2_Transitions=
{
  iteration: 6,
  time: 10
};
/**
  * Transition is a class to make all kind of animations.
  * it gets returned with the methode createTransition on all HTML elements.
  * @param element is the element which shall be animated
  * @param iteration a default value in percents, if not set it takes the global ( O2_Transition.iteration )
  * @param callbefore optional
  * @param callback optional
  * @param transformer a function to correct the current iteration. The default is: function(x){return 1}. Should return a value between 0 and 1 for 0 to 100.
  * @param method default is: reset. Possible value is: flip
  * @constructor
  */
function Transistion(element, iteration, callbefore, callback, transformer, method)
{
  this.effects = new TransitionEffects();
  var properties=[];
  var self=this;
  var current=0;
  var interval=40;
  var run=false;
  var startTime=null;
  var stragedyInt=function(prop)
  {
    return (prop.from+(Math.round(prop.delta*current/100)))+prop.unit;
  };
  var stragedyFloat=function(prop)
  {
    return (prop.from+prop.delta*current/100);
  };
  var currentStragedy=null;
  var onStep=function()
  {
    var last=false;
    var temp=null;
    startTime=temp;
    if((current+=self.iteration*self.transform(current))>=100)
    {
      current=100;
      clearInterval(interval);
      last=true;
    };
    var i=0, prop=null;
    
    for ( ; prop=properties[i]; i++)
    {
      self.styles[prop.property]=prop.stragedy(prop);
    }
    if(last)
    {
      switch(self.method)
      {
        case 'flip':
          var i=0, prop=null, temp=0;
          for ( ; prop=properties[i]; i++)
          {
            temp=prop.to;
            prop.to=prop.from;
            prop.from=temp;
            prop.delta=prop.to-prop.from;
            
          }
          current=0;
          run=false;
          break;
        case 'reset':
          current=0;
          run=false;
          break;
      }
      if(self.callback) self.callback();
    }
  }
  /** a refernce to the style object from the element to animate */
  this.styles=element.style;
  /** a refernce to the the element to animate */
  this.reference=element;
  /** the default iteration value in percents. if not set it's O2_Transitions.iteration*/
  this.iteration=iteration||O2_Transitions.iteration;
  	/** optional */
  this.callbefore=callbefore||null;
  /** optional */
  this.callback=callback||null;
  /** a function to correct the current iteration. The default is: function(x){return 1}. Should return a value between 0 and 1 for 0 to 100. */
  this.transform=transformer||this.effects.linear;
  /** method default is: reset. Possible value is: flip
  * reset means at the end the transition will be prepared to make a other transition.
  * flip means all from-to paires will be exchanged at the end*/
  this.method=method||'reset';
  /** methode to add a property to the transition
  * @param property the property to animate ( right now dimesions and opacity supported, not jet colors )
  * @param from startvalue
  * @param to endvalue
  * @return the transition
  */
  this.add=function(property, from, to)
  {
    var strat=null, unit='';
    if(property=='opacity')
    {
      strat=stragedyFloat;
    }
    else
    {
      strat=stragedyInt;
      unit=from.replace(/-?\d*/g,'');
      from=parseInt(from);
      to=parseInt(to);
    }
    var delta=to-from;
    var pointer_here=null;
    if(pointer_here=this.getProperty(property));
    else pointer_here=properties[properties.length]={};
    pointer_here.property=property;
    pointer_here.unit=unit;
    pointer_here.from=from;
    pointer_here.to=to;
    pointer_here.delta=delta;
    pointer_here.stragedy=strat;
    return this;
  }
  /** methode to get a property from the transition.
  * @param property
  * return the property object
  */
  this.getProperty=function(property)
  {
    var i=0, pointer=null;
    for( ; pointer=properties[i]; i++)
    {
      if(property==pointer.property) return pointer;
    }
    return null;
  }
  /** methode to check if the transition is running 
  * return boolean
  */
  this.checkRun=function()
  {
    return run;
  }
  /** methode to strat the transition */
  this.execute=function()
  {
    if(!run)
    { 
      run=true;
      if(self.callbefore) self.callbefore();
      interval=setInterval(onStep, O2_Transitions.time);
    }
  }
};

(function()
{
  this.createTransition=function(iteration, callback, transformer, method)
  {
    return new Transistion(this, iteration, callback, transformer, method); 
  };
}).apply(HTMLElement.prototype);


function TransitionEffects()  
{
  var speed = 1;
  this.sine = function(x)
  {
    var sin=Math.sin;
    var pi=Math.PI;
    var start=.1;
    return ((sin((x)/100*pi))+start)*speed; 
  }
  
  this.linear = function(x)
  {
    return 1*speed;
  }
  
  this.setSpeed = function(s){
    speed = s;
  }
  
}